#include "generator/DataSource.hh"
#include "generator/DampedSine.hh"
#include "generator/FrameData.hh"
#include "generator/GaussBurst.hh"
#include "generator/Sinc.hh"
#include "generator/SineData.hh"
#include "generator/SinGauss.hh"
#include "generator/WhiteNoise.hh"
#include <iostream>

using namespace std;

//======================================  Static name parser
generator::DataSource*
generator::DataSource::parse(const std::string& id) {
    enum {
      kInName,
      kInOp,
      kFndEq,
      kCparen
    } state(kInName);
    DataSource* rc(0);

    bool inquote(false), syntax(false), fnop(false);
    int argop(0);
    string SrcName, argName, argValue;
    const char* ptr = id.c_str();
    while (*ptr && !syntax) {
        char c = *ptr++;
	if (inquote) {
	    argValue += c;
	    inquote = (c != '\'' && c != '\"');
	} else {
	    switch (c) {
	    case '(':
	        if (state == kInName) {
		    if (SrcName == "SinGauss") {
		        rc = new SinGauss;
		    } else if (SrcName == "WhiteNoise") {
		        rc = new WhiteNoise;
		    } else if (SrcName == "DampedSine") {
		        rc = new DampedSine;
		    } else if (SrcName == "FrameData") {
		        rc = new FrameData;
		    } else if (SrcName == "Sine") {
		        rc = new SineData;
		    } else if (SrcName == "Sinc") {
		        rc = new Sinc;
		    } else if (SrcName == "GaussBurst") {
		        rc = new GaussBurst;
		    }
		    syntax = !rc;		    
		    state = kInOp;
		} else if (state == kFndEq) {
		    argValue += c;
		    ++argop;
		    fnop = true;
		} else {
		    syntax = true;
		}
		break;

	    case '=':
	        if (argop) argValue += c;
	        else       state = kFndEq;
	        break;

	    case ')':
	    case ',':
	        if (argop) {
		    argValue += c;
		    if (c == ')') argop--;
	        } else if (state == kFndEq) {
		    rc->setParameter(argName, argValue);
		    argName.clear();
		    argValue.clear();
		    fnop = false;
		    if (c == ',') state = kInOp;
		    else          state = kCparen;
		} else {
		    syntax = true;
		}
	        break;

	    case '\'':
	    case '\"':
	        argValue += c;
	        if ( state == kFndEq ) inquote = true;
		else                   syntax  = true;  
		break;

	    default:
	        switch (state) {
	        case kInName:
		    SrcName  += c;
		    break;
		case kInOp:
		    argName  += c;
		    break;
		case kFndEq:
		    argValue += c;
		    break;
		default:
		    syntax = true;
		}
	    }
	}
    }

    //----------------------------------  Check for syntax error
    if (argop || state == kInOp) {
        cerr << "Mismatched parentheses in " << id << endl;
	syntax = true;
    } else if (inquote) {
        cerr << "Mismatched quotes in " << id << endl;
	syntax = true;
    } else if (syntax) {
        cerr << "Syntax error in " << id << endl;
	syntax = true;
    }
    if (syntax && rc) {
        delete rc;
	rc = 0;
    }

    //-----------------------------------  Record definition
    if (rc) {
        rc->mSeries.setName(id.c_str());
    }
    return rc;
}
