#include "PTChan.hh"
#include "Dacc.hh"
#include "DVector.hh"
#include "DecimateBy2.hh"
#include "Trend.hh"
#include "DVecType.hh"

#include <iostream>
#include <fstream>
#include <iomanip>
#include <algorithm>
#include <vector>
#include <cassert>

using namespace std;
using std::oct;

//--------------------------------------- Include C headers 
 
#include "detect.h"
#include "ligoconfig.h"
#include "ligocont.h"
#include "ligostd.h"

//--------------------------------------- Include C functions

extern "C" {
void Cont_setup(con_sto *);
int *E_buffer(int, int);
unsigned char E_create(struct detect_info *, int, int, const char *,
                        int *, unsigned char (*)(), double *);
unsigned char E_detect(struct detect_info*, struct s_onset *, int *, const int);
unsigned char E_filter(struct detect_info*);
unsigned char Event();
unsigned char FfirBB();
void Onset(int,int,int,int,int,int,int,int,int,int,
	   struct detect_info*, struct s_onset*, int*, const int);
  void Onsetq( struct detect_info*, struct s_onset*, int*, const int);
unsigned char P_one(int);
}

//=========================================== Channel constructor
PTChan::PTChan(void)
  : mSps(0), mThresh(0), mOnsetN(0), mNTotal(0), mDacc(0), mTS(0), mDec(3,1)  
{
}

//======================================= Secondary constructor
PTChan::PTChan(const char* name) 
  : mSps(0), mThresh(0), mOnsetN(0), mNTotal(0), mChannel(name), mDacc(0), mTS(0),
      mDec(3,1)
{
} 

//======================================= Copy constructor
PTChan::PTChan(const PTChan& x)
  : mSps(x.mSps), mThresh(x.mThresh), mOnsetN(x.mOnsetN), mNTotal(x.mNTotal), mChannel(x.mChannel),      mDacc(x.mDacc), mTS(x.mTS),mDec(x.mDec)  
{
}

//======================================= Destructor
PTChan::~PTChan(void) {
  if (mDacc && mTS) mDacc->rmChannel(mChannel.c_str());
}

//======================================= Get channel name
const char*
PTChan::getChannel(void) const {
  return mChannel.c_str();
}

//======================================= Get channel sampling rate
void
PTChan::setSps(long int samSec) {
  mSps = samSec;
}

//======================================= Specify a Dacc, request the channel
void
PTChan::setup(Dacc& In) {
    mDacc = &In;
    if (!In.isChannelRead(mChannel.c_str())) {
        In.addChannel(mChannel.c_str(), 1, &mTS);
    }

//---------------------------- Set parameters according to channel type
 
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++ Seismic channels

    if (mSps == 256) {      // Seismic channels

//---------------------------- Create data buffer: current frame + lookback
int maxDataPoints = 130;
int maxLookBack   = 40;
int * mDataArray  = E_buffer(maxDataPoints, maxLookBack);

//---------------------------- Arrange to pass filter coeeficients
int maxCoef = 37;
coefArray = new double[maxCoef];
double * mCoefPtr = (double *)coefArray;

//------------------------------------- Create "Event detector" 
    E_create(&detect,
           36,                // Lookback for filter 
           256,               // Channel type (samples per second: sps)
           mChannel.c_str(),  // Channel name  
           mDataArray,        // Address of data buffer
           FfirBB,            // Filter to be used (5 to 45 Hz)
           mCoefPtr);         // Address of filter coefficients
     detect.sam_sec = 128;
     detect.nCoef   = 37;

//------------------------------------- Read in filter coefficients
     //Aug03 filter:- Seis37.dat, 5-45 Hz
assert(coefArray!=0);

  ifstream inCoefFile("Seis37.dat", ios::in);
  if(!inCoefFile) {
    cerr << "Seismic coefficient file could not be opened" << endl;
    exit(1);
  }

  for (int i = 0; i < maxCoef; i++) {
    inCoefFile >> coefArray[i];
  }

//------------ Load event detection parameters into the continuity structure

  setallparams(&detect,    // setallparams is a macro in ligocont.h
  
        1,       // Filhi (decimal) -- filhi << fillo          
	32,      // Fillo (decimal) -- 0 << fillo < iwin       
        64,     // Iwin (decimal) -- don't wrap around E_B    
	4,       // N_hits (decimal) -- 4 or 5 recommended      
        020,     // Xth1 (octal) -- xth1 > xth2, xth1 <= 0377  
        015,     // Xth2 (octal) -- xth2 > xth3, 015-017 rec.  
	012,     // Xth3 (octal) -- recommended value 015         
        015,     // Xthx (octal) -- ~013 <= xthx <= 0377, 015-030 rec.   
	0,       // Def_tc (decimal) -- 1/2 nominal signal period, in ns  
	256,     // Wait_blk (decimal) -- P-Ts for estimating period  
	16);     // Val_avg (decimal) -- 1 <= val_avg <= 16 [background]   

//-------------------- [Display parameters on console and] decimate
 
          mDec.setDecimation(1,1); 

    }                       // End seismic setup

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 2K channels

    if (mSps == 2048) {     // Auxilliary channels 

//---------------------------- Create data buffer: current frame + lookback
int maxDataPoints = 1000;
int maxLookBack   = 50;
int * mDataArray  = E_buffer(maxDataPoints, maxLookBack);

//---------------------------- Arrange to pass filter coeeficients
int maxCoef = 50;
coefArray = new double[maxCoef];
double * mCoefPtr = (double *)coefArray;

//------------------------------------- Create "Event detector" 
    E_create(&detect,
           41,                // Lookback for filter 
           2048,              // Channel type (samples per second: sps)
           mChannel.c_str(),  // Channel name  
           mDataArray,        // Address of data buffer
           FfirBB,            // Filter to be used 15 to 75 Hz)
           mCoefPtr);         // Address of filter coefficients
     detect.sam_sec = 256;
     detect.nCoef   = 41;

//------------------------------------- Read in filter coefficients
     //Aug03 filter: Accel41.dat, 15-75 Hz
assert(coefArray!=0);

  ifstream inCoefFile("Accel41.dat", ios::in);
  if(!inCoefFile) {
    cerr << "Auxilliary coefficient file could not be opened" << endl;
    exit(1);
  }

  for (int i = 0; i < maxCoef; i++) {
    inCoefFile >> coefArray[i];
  }

//------------ Load event detection parameters into the continuity structure

  setallparams(&detect,    // setallparams is a macro in ligocont.h
  
        1,       // Filhi (decimal) -- filhi << fillo          
	32,      // Fillo (decimal) -- 0 << fillo < iwin       
        64,      // Iwin (decimal) -- don't wrap around E_B    
	4,       // N_hits (decimal) -- 4 or 5 recommended      
        020,     // Xth1 (octal) -- xth1 > xth2, xth1 <= 0377  
        015,     // Xth2 (octal) -- xth2 > xth3, 015-017 rec.  
	012,     // Xth3 (octal) -- recommended value 015         
        015,     // Xthx (octal) -- ~013 <= xthx <= 0377, 015-030 rec.   
	0,       // Def_tc (decimal) -- 1/2 nominal signal period, in ns  
	300,     // Wait_blk (decimal) -- P-Ts for estimating period  
  	16);     // Val_avg (decimal) -- 1 <= val_avg <= 16 [background]   

//--------------------------   [Display parameters on console and] decimate
 
          mDec.setDecimation(3,1); 
    }                       // End setup for auxilliary channels

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ LASER Channels

    if (mSps >= 4096) {    // Laser channels 

//---------------------------- Create data buffer: current frame + lookback
int maxDataPoints = 1030;
int maxLookBack   = 55;
int * mDataArray  = E_buffer(maxDataPoints, maxLookBack);

//---------------------------- Arrange to pass filter coeeficients
int maxCoef = 55;
coefArray = new double[maxCoef];
double * mCoefPtr = (double *)coefArray;

//------------------------------------- Create "Event detector" 
    E_create(&detect,
           51,                // Lookback for filter 
           16384,             // Channel type (samples per second: sps)
           mChannel.c_str(),  // Channel name  
           mDataArray,        // Address of data buffer
           FfirBB,            // Filter to be used (100 to 300 Hz)
           mCoefPtr);         // Address of filter coefficients
     detect.sam_sec = 1024;
     detect.nCoef   = 51;

//------------------------------------- Read in filter coefficients
assert(coefArray!=0);

  ifstream inCoefFile("Hann51_70_300.dat", ios::in);
  if(!inCoefFile) {
    cerr << "Laser coefficient file could not be opened" << endl;
    exit(1);
  }

 
//------------ Load event detection parameters into the continuity structure

  setallparams(&detect,    // setallparams is a macro in ligocont.h
  
        1,       // Filhi (decimal) -- filhi << fillo          
	32,      // Fillo (decimal) -- 0 << fillo < iwin       
        64,     // Iwin (decimal) -- don't wrap around E_B    
	4,       // N_hits (decimal) -- 4 or 5 recommended      
        020,     // Xth1 (octal) -- xth1 > xth2, xth1 <= 0377  
        015,     // Xth2 (octal) -- xth2 > xth3, 015-017 rec.  
	012,     // Xth3 (octal) -- recommended value 015         
        015,     // Xthx (octal) -- ~013 <= xthx <= 0377, 015-030 rec.   
	0,       // Def_tc (decimal) -- 1/2 nominal signal period, in ns  
	128,     // Wait_blk (decimal) -- P-Ts for estimating period  
	16);     // Val_avg (decimal) -- 1 <= val_avg <= 16 [background]   

//---------------------------- [Display parameters on console and] decimate

 for (int i = 0; i < maxCoef; i++) {
    inCoefFile >> coefArray[i];
  }
  if(mSps==16384){
     mDec.setDecimation(4,1);
  } else if(mSps==8192)
     mDec.setDecimation(3,1);
  } else {
     mDec.setDecimation(2,1);
  }

}                             // End setup for laser channels

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                             
//======================================= Get TSeries data
const TSeries&
PTChan::refTSeries(void) const {
    if (mTS) return *mTS;
    else if (!mDacc) throw runtime_error("TSeries not available");
    return *mDacc->refData(getChannel());
}

//======================================= Set decimation rate
void
PTChan::setDecimation(int N, int F) {
}

//======================================= Apply decimation
TSeries
PTChan::apply(const TSeries& in) {
return *mTS = in;
}

//======================================= Float TSeries
void
PTChan::reset(void) {
  if (mTS)  mTS->Convert(DVecType<float>::getDataType());
}

//======================================= Specify enable condition
void
PTChan::setEnableCond(const string& str) {
    mEnableCond = str;

}
//======================================= Find the glitches
bool
PTChan::findit(const int maxOnset, s_onset* oPtr) {

   bool Dropout = false;
   bool Glitch = false;
   mTrigger = false;
   int nOnset = 0;

   const TSeries* ts = &refTSeries();
   int *oarray;                 // Address of subarray for new data
   oarray = &detect.indatar[detect.lbksize];

//-------------------------------------- Check data for continuity and decimate

    if (!mLast){
      mLast = (*ts).getStartTime();
    }
    if (!(mLast == (*ts).getStartTime())){
      Dropout = true;
    }
    if (!mDec.isDataValid(*ts)) {
          mDec.reset();
      }
          TSeries t(mDec.apply(*ts));
          size_t N = t.getNSample();
      	  t.getData(N, oarray);

//-------------------------------------- Get start time for frame

        mtStart = t.getStartTime();
    int gpssec  = mtStart.getS();
    int gpsns   = mtStart.getN(); 

//--------------------------------------- Complete initialization
    detect.datapts    = N;           // Data points per record
    detect.startt.sec = gpssec;      // Start time, seconds
    detect.startt.ns  = gpsns;       // Start time, nanoseconds

//------------------------------------- Filter Data & Launch Event Detector
 
    if(detect.datapts){
       E_filter(&detect);            // Filter decimated data

       E_detect(&detect, oPtr, &nOnset, maxOnset);   
                                     // Call 'event detector': traffic cop
       nOnset = *(&nOnset);

//------------------------------------- Deal with data dropouts if necessary

    if (Dropout && nOnset){
      int counter = nOnset;
      for (int j=0; j<nOnset; j++){
        float ns2s = (float)oPtr[j].o_ns;
        if(oPtr[j].o_secs == gpssec && ns2s < mSettle){
          oPtr[j].o_maxamp = 0.;
          counter = counter - 1;
        }
      }
      nOnset = counter;
    }
      mLast = t.getEndTime();

      int realOnset = 0;

      if (nOnset){
         for (int j=0; j<nOnset; j++){
	   if(oPtr[j].o_maxamp >= mThresh){
	      realOnset++;
              Glitch = true;
              mOnsetN = realOnset;
              mNTotal += realOnset;
              mObserved += t.getInterval();
	   }

	 }

      }

    }   

    
    mTrigger = Glitch;
   
return Glitch; 

}
