#define MAINDEF		    

/* Make "detect" allocate storage for all global variables              *
 *in this module - see detect.h                                         */
#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: BOOL E_detect(detector,Ptr,onsetNum,onsetMax)             * 
 *                      - Event detect a data record		        *	  
 *----------------------------------------------------------------------*
 *	Arguments:	struct detect_info *detector - information for	*
 *					running the detector		*
 *----------------------------------------------------------------------*
 *	Returns:	TRUE - Current record is within the interval	*
 *				of an event.				*
 *			FALSE - Current record is not within the 	*
 *				 interval of an event.			*
 *									*
 *			      ---The interval of the event---		*
 *			Upon declaring an event, E_detect will remain	*
 *			 TRUE for a minium of NOR_OUT x wait_blk 	*
 *			 samples, where NOR_OUT = 4 and 		*
 *			 wait_blk = 1014 or 507.  It will remain true	*
 *			 longer than NOR_OUT x wait_blk if a retrigger	*
 *			 occurs in the coda of the event.		*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	E_detect is the main driver for the event detect algorithm.	*	
 *	 The function of E_detect is to call routines that 		*
 *		(1)  Find the amplitudes and associated times		*
 *		      of the filtered data, P_one.			*
 *		(2)  Process these amplitudes and associated 		*
 *		      times to search for seismic signals, Event.	*
 *		(3)  Cause output of the parameters of the		*
 *		      detected signal, Onsetq.				*
 *									*
 *	The main (calling program) passes to E_detect the address	*
 *	 of a structure.  Although this structure (detector) con-	*
 *	 tains several other variables and constants, the only		*
 *	 ones needed by E_detect are:					*
 *	 	(1)  sample rate x 1000 of the digital data. (This	*
 *	 	      multiplication is done so that VLP data of	*
 *		      the China system can be detected, if one so	*
 *		      desires.)						*
 *		(2)  the number of data points that will be processed.	*
 *		(3)  the time (second, nanosecond) of the first data	*
 *		      point of the current record (the data that will  	*
 *		      be processed).		                        *
 *		(4)  the address of the first data point to be pro-	*
 *		      cessed.  (The address of the array of 64 bit	*
 *		      data that will be processed.)
 *		(5)  the address of the list ( a structure) of 		*
 *		      variables and constants that must be maintained	*
 *		      for each detector.  The parameters of the detec-	*
 *		      tor (filhi, fillo, xth1,etc.) are included in	*
 *		      this list and must be loaded into the list by	*
 *		      the user.						*
 *		(6)  the channel name                              	*
 *									*
 *	E_detect returns TRUE (the value 1) if any the data of the	*
 *	 current record are within the interval of the event--		*
 *	 "the interval of the event" is a programmable feature,		*
 *	 as explained above.  Otherwise E_detect returns False (the	*
 *	 value 0).							*
 *----------------------------------------------------------------------*/

BOOL E_detect(detector, oPtr, nOnsetPtr, maxOnset)
struct detect_info *detector;
struct s_onset * oPtr; 
int * nOnsetPtr;
/* const int maxOnset; */

{
	LONG *ldp,tdat;
	BOOL tapewrite;            /* See 'Explanation of the Variables' */

	con_ptr = detector->incontd;      /* User's continuity structure */

	tapewrite = FALSE;

	ldp = detector->indatar;
							 /* Note 1 */
/*--------------Associate the data with a record number-------------------*/

	con_ptr->cur_rec++;
	if (con_ptr->cur_rec >= CUR_MAX)
	    con_ptr->cur_rec = 0;
/*-- Provision for changing sampling rate during processing removed here---*/

        con_ptr->sam_sec = detector->sam_sec;
	con_ptr->sam_ch = detector->datapts;

	    			       /* # of data points to be processed */
        con_ptr->ns_sam = 1000000000L/con_ptr->sam_sec;
        con_ptr->ns_sec = 1000000000L;

	etime.sec = detector->startt.sec;
	etime.ns = detector->startt.ns;

/*--------etime will be changed when events are detected - backup--------*/

	htime.sec = etime.sec;
	htime.ns = etime.ns;

/*--------------Arrange for output data to go to a file-------------------*/
	/*         con_ptr->ponset = FALSE;         */
        /*         con_ptr->onsproc = refOnsetData; */


/*--------------------------Process the input data------------------------*/

	for (sam_no = 0; sam_no < con_ptr->sam_ch; sam_no++) {

							/* Note 2 */
/*----------P-T detected, p_one=TRUE; event declared, Event=TRUE-----------*/
/*			Send input data to P_one			   */

	    tdat = *ldp;

	    if (P_one(tdat) && Event())
	        Onsetq(detector, oPtr, nOnsetPtr, maxOnset);

	    ldp++;
							/* Note 3 */
	    if (con_ptr->itc) {	       /* is data within interval of event? */

	        Count_dn();	       /* decrements itc */

	        tapewrite = TRUE;
	    }
	}

	return(tapewrite);
}

#ifdef JNMCOMMENT
/***************************************************************************

	    				NOTES

	    [1]	In P_one, a peak or trough is not declared until one sample
	    	 past the maximum or minimum.  Thus, should the last sample
	    	 of a record be a maximum or minimum, neither will be detected
	    	 until the next record has been read.  We need to keep track
	    	 of in which record the data occur to cope with this 
	    	 situation.

	    [2]	The input time series is sent to routine P_one which 
	    	 calculates the signed amplitudes (peak-to-trough diff-
	    	 erences) and their corresponding times.  P_one calls
	    	 P_two which estimates the statistical dispersion of
	    	 the amplitudes and sets the thresholds.  When a peak
	    	 or trough has been found by P_one, P_one returns TRUE
	    	 and Event is called to determine whether or not the
	    	 current amplitude might be part of a signal.  When
	    	 a signal is declared by Event, it returns TRUE, and
	    	 Onsetq is called for further processing of the signal.
	    	 Onsetq calls Onset which outputs information about the
	    	 signal.
 
	    [3]	The counter itc is greater than zero if the current sample
	    	 is within the interval of the event.  Routine Count_dn
	    	 decrements itc.


*******************************************************************************

	    		EXPLANATION OF THE VARIABLES

	    con_ptr	--  global variable which allows all event 
			     detector to access the continuity structure.

	    CUR_MAX	--  the upper bound of the arbitrary record numbers.
	    		     The record numbers are reset here.

	    cur_rec	--  an index that keeps track of the record
	    		     in which the amplitudes of the input time
	    		     series occur.

	    itc		--  a counter that is greater than zero if the
	    		     current data sample is within the interval
	    		     of the event. Initialized when an event is
	    	   	     detected to NOR_OUT.  Decremented in Count_dn.

	    incontd	--  the slot in the detect_info structure
			     containing this detectors continuity structure

	    ldp		--  a pointer that is initialized to the address
	    		     of the first seismic data sample of the 
	    		     current record.

	    new_onset	--  a flag set=TRUE if a signal was declared in
	    		     the current record.  Set=FALSE otherwise.

	    sam_ch	--  samples per channel of the seismic data record.

	    sam_no	--  sample number of seismic data of the current
	    		     demultiplexed record.

	    s_r_x_1000	--  adjusted sample rate to permit processing VLP
	    		     data that is less than 1 sample/sec.
	    		     (sam_sec x 1000).

	    samrte  	--  adjusted sample rate to permit processing VLP
	    		     data that is less than 1 sample/sec.
	    		     (sam_sec x 1000).

	    sam_sec	--  sample rate of the current seismic data record
	    		     (samples per second).
	    
	    tapewrite	--  a flag set=TRUE if any data of the current record
	    		     is within the interval of the event.  Set=FALSE
	    		     otherwise.  E_detect returns the value of tape-
	    		     write.  The concept here is that the calling 
	    		     routine might wish to write data to tape during,
	    		     and prior to, the interval of the event.
	    		    
*************************************************************************/
#endif
