// implement simple event class
// NOTE THAT THIS CLASS DOES NOT ALLOCATE ANY MEMORY DYNAMICALLY

// C++ includes
#include "stdio.h"
#include <iostream>
#include <vector>

// dmt includes
#include "TSeries.hh"
#include "FSeries.hh"
#include "Dacc.hh"
#include "Interval.hh"
#include "Time.hh"
#include "DVector.hh"
#include "TSeries.hh"

// slope search includes
#include "Event.hh"

using namespace std;

// default constructor makes an empty event

Event::
Event(void) {
  _in_use = false;
  _raw_available = false;
  _event_display = 0x0;
  _raw_event_display = 0x0;
  _event_time = (Time)0;
  _event_duration = (Interval)0.0;
  _event_amplitude = 0.0;
  return;
}

// constructor that maintains data at location ptseries

Event::
Event( TSeries* ptseries, Time event_time, 
       Interval event_duration, double event_amplitude ) {
  setup(ptseries,event_time,event_duration,event_amplitude);
}

// constructor that maintains data at location ptseries and
// raw data at location prawtseries
Event::
Event( TSeries* ptseries, TSeries* prawtseries, Time event_time,
       Interval event_duration, double event_amplitude ) {
#ifdef DEBUGSLOPE
  cout << "running setup" << endl;
#endif
  setup(ptseries,prawtseries,event_time,event_duration,event_amplitude);
}

// copy constructor that keeps same pointer to time series

Event::Event(const Event& y) {
  _event_display = y._event_display;
  _raw_event_display = y._raw_event_display;
  _event_time = y._event_time;
  _event_duration = y._event_duration;
  _event_amplitude = y._event_amplitude;
  _in_use = y._in_use;
  _raw_available = y._raw_available;
}

// assignment operator keeps same pointer to time series

Event&
Event::operator=(const Event& y) {
  if(this != &y) {
    _event_display = y._event_display;
    _raw_event_display = y._raw_event_display;
    _event_time = y._event_time;
    _event_duration = y._event_duration;
    _event_amplitude = y._event_amplitude;
    _in_use = y._in_use;
    _raw_available = y._raw_available;
  }
  return *this;
}

Event::
~Event(void) {
  _in_use = false;
  _raw_available = false;
}

void
Event::
setup( TSeries* ptseries, Time event_time, 
       Interval event_duration, double event_amplitude ) {
  _event_display = ptseries;
  _event_time = event_time;
  _event_duration = event_duration;
  _event_amplitude = event_amplitude;
  _in_use = true;
}

void 
Event::
setup( TSeries* ptseries, TSeries* prawtseries, Time event_time,
       Interval event_duration, double event_amplitude ) {
#ifdef DEBUGSLOPE
  cout << "setting up event with raw data" << endl;
#endif
  _raw_available = true;
  _raw_event_display = prawtseries;
  setup(ptseries,event_time,event_duration,event_amplitude);
  return;
}
  
double
Event::
getAmplitude(void) {
  return _event_amplitude;
}

Time
Event::
getTime(void) {
  return _event_time;
}

Time
Event::
getViewerStartTime(void) {
  return _event_display->getStartTime();
}

Interval
Event::
getViewerDuration(void) {
  return ( _event_display->getEndTime() -
	   _event_display->getStartTime() );
}

Interval
Event::
getDuration(void) {
  return _event_duration;
}

TSeries
Event::
getDisplay(void) {
  return *_event_display;
}

TSeries*
Event::
getDisplayPointer(void) {
  return _event_display;
}

void
Event::addRawDisplay(TSeries* pts) {
  _raw_available = true;
  _raw_event_display = pts;
  return;
}

TSeries* 
Event::getRawDisplayPointer(void) {
  return _raw_event_display;
}

TSeries
Event::getRawDisplay(void) {
  return *_raw_event_display;
}

bool
Event::rawDataAvailable(void) {
  return _raw_available;
}

void
Event::textSummary(std::ostream& os) {
  os << "#TIME: " << getTime() << endl;
  os << "#AMPLITUDE: " << getAmplitude() << endl;
  os << "#DURATION: " << getDuration() << endl;
  double* data = new double [ _event_display->getNSample() ];
  _event_display->getData(_event_display->getNSample(), data);
  for(int i=0; i<(int)_event_display->getNSample(); ++i) {
    os << i << "\t" << data[i] << endl;
  }
  delete [] data;
  return;
}

void
Event::shortTextSummary(std::ostream& os) {
  os << (getTime()).getS() << "\t" 
     << (getTime()).getN() << "\t"
#ifdef DEBUGSLOPE
     << getDisplayPointer() << "\t"
#endif
     << getAmplitude() << endl;
  return;
}

bool
Event::
operator>
(const Event& y) const {
  return ( _event_amplitude > y._event_amplitude );
}

bool
Event::
operator<
(const Event& y) const {
  return ( _event_amplitude < y._event_amplitude );
}






