#ifndef PARSER_HH
#define PARSER_HH

#include <string>
#include <stream.h>
#include <fstream>
#include <vector>

using namespace std;

//________________________________________________________________________________________________________

//createtokens takes a string that has been read in from configfile(line)
//and parses the line wereever there is whitespace or and = sign
//tok_num is an int variable that readconfig use to pass to other functions
//when they search the vector for a specific string
//
//note all whitespace quotation signs and = signs have been dropped in the returned vector
//
//returns a vector filled with strings from the line 

vector<string> createtokens(string line,int &tok_num);

//wordsearch takes in a string that will be searched for, a vector of strings to be searched
//and an int span which correspons to tok_num from create tokens which tells wordsearch
//how many times to run the search
//returns true if the word is found, false if not.  

bool wordsearch(string word,vector<string> stuff, int &span,int start);


//GetPositionInfo 
//num_var represents the number of variable to be searched for like 3 channel etc.
//start_search is a variable that is needed for the wordsearch program so that it knows where to start
//vector_length is the length of the vector being searched by wordsearch
//tokens is the vector to be searched
//variable is the name of thing to be searched for (e.g. Channel, or Band)
//
//GetPositionInfo returns a vector of int with the position of each variable in the string vector
//so if there are 3 channels to be found the returned vector[0]= position of word Channel1 in string vector
//this is made to speed up searching for variables

vector<int> GetPositionInfo(int num_var,int start_search,int vector_length,vector<string> tokens,string variable);

//SetEndSearch resets the varible in readconig that is used so only a certain part of the vector is searched
//total is the total size of the vector being searched
//variablenumber is the number of variable( # of channels etc.)
//loopnumber is how many time the loop has been gone through ( i=2)
//position is the that holds the starting pts for the bands or channels
//this needs to be used whenever there might be a vector that doesnt have enought elements such as a BandPosition vector
//NOTE THIS MUST BE RUN INSIDE THE LOOP IN WHICH YOU ARE USING

int SetEndSearch(int total,int variablenumber, int loopnumber,vector<int> position);

//total is a vector of string representing the whole config file. 
//using start and ending GetRequestedData make a new vector<string> that only has the information on the ifo specified 
//by the user
vector<string> GetRequestedData(vector<string> total,int start, int ending);

//readconfig reads in a .conf file from the local directory or if not from a base directory
//spcified in the program
//takes in a string that is the name of the file to be read in.  
//ifoname is the interferometer being used H1, H2 etc
//since the config file is set up in a very specific way, readconfig returns a vector of strings
//that are searched in a very specific way.  if a certain string is found then SpecMon does 
//a certain action
//example returned vector
//info[0] = "1" 		(Number of Channels to be monitored)
//info[1] = "Channel1" 		(Name of Channel 1)
//info[2] = "3" 		(Number of Bands to be monitored in Channel)
//info[3] = "Band1"		(Name of Band 1)
//info[4] = 1234		(float freqlo)
//info[5] = 4321 		(float freqhi)
//info[6] = 321			(float hi_power)
//info[7] = 123			(float lo_power)
//info[8] = 2			(number of threshold settings)
//info[9] = 123			(thresh1_lo)
//info[10] = 12			(thresh1_hi)
//info[11] = 134		(thresh2_lo)
//info[12] = 23			(thresh2_hi)
//this is then repeated for each band in the channel and then for the number of channels

vector<string> readconfig(string filename,string ifoname);


/**************************************
**                                   **
**		READCONFIG           **
**				     **
**************************************/

vector<string> readconfig(string filename,string ifoname)
{
  ifstream configfile;
  int linenumber =0;
  int tok_num= 0;			//number of words parsed so far
  string line;				//line that is read in
  string problem;			
  vector<string> tokens;	//stores the strings being parsed
  vector<string> info;		//info is a vector that has only the relevant values that are to be passed
  int infocount = 0;
  int mDebug =0;
   
  configfile.open(filename.c_str());	//open config file

  if (configfile.eof())
    {
      cerr<< "Config_File.readconfig(): error in opening file named "<< filename<<endl;
      exit(EXIT_FAILURE);
    }
	
  while(!configfile.eof())
    {
      //Read in first line
      getline(configfile, line);	//max number is 1024 chars in the string since
      
      //skip lines that are comments, ones beginning with "#" or "*"
      if (strchr("#", line[0]) != NULL || strchr("*", line[0]) != NULL)
	continue;

      if (line.length() > 0)
	{
	  ++linenumber;
	  
	  //Break line into separate strings that are stored in a vector of strings
	  //idea was based off of some unused code in OperStateCond.cc by David Chin
	  
	  int temp=0;	//number of parsed stings in the line
	  
	  if(linenumber ==1)
	    {
	      tokens = createtokens(line,temp);//array of parsed strings
	    }else
	      {
         	vector<string> temptokens = createtokens(line,temp);
         	
         	for(int i=0; i<temp;++i)
		  {
		    tokens.insert(tokens.end(),temptokens[i]);
		  }
	      }
	  tok_num = (tok_num + temp);   
	}  
    }
       
  string flag = tokens[0];
    
  //check to see if everything is all right when the config file is read by outputting parsed words
  if(mDebug == -1)
    {
      for(int i=0; i<tok_num;++i)
    	{
	  cout<<i<<" "<<tokens[i]<<endl;
    	} 
    }
   
  //___________________________________________Find which ifo is being used___________________________________    
    
  //Find out where we are and then throw out the rest of the information.  
  int start =0;
  int wordnumber = tok_num;
  int ending = 0;
    
  if(wordsearch(ifoname,tokens,wordnumber,start))
    {
      start = (wordnumber +1);
    }else
      {
    	problem = "Information for "+ ifoname + " is not specified in configuration file: Please fix and restart";
    	cerr<<problem<<endl;
    	throw runtime_error(problem);
      }
    
  wordnumber = tok_num;
    
  if(wordsearch("H1",tokens,wordnumber,start) ||wordsearch("H2",tokens,wordnumber,start) || wordsearch("L1",tokens,wordnumber,start))
    {
      ending = wordnumber;
      //cout<<"Ending = "<<ending<<endl;
    }else{
      ending = tok_num;
    }
  
  // cout<<tokens[start]<<tokens[ending]<<endl;
  
  tokens = GetRequestedData(tokens,start,ending);
  
  if(mDebug >9)
    cout<<"Getting requested data"<<endl;    	
  
  tok_num = ((ending - start));
  //cout<<tok_num<<endl;
  
  if(mDebug== -5 || mDebug >8)
    {
      for(int i = 0; i<tok_num; ++i)
    	{
	  cout<<"Tokens "<< i <<" = " <<tokens[i]<<endl;
    	}
    }
 	
  //__________________________________________________Get information on the ifo_________________________________
	
	//CONVERT ALL OF THE STRINGS TO FLOATS AND INTS IN SPECMON.CC

  wordnumber= tok_num;	//variable to be passed by reference to wordsearch that will locate string place in the array,
  //also used in conjunction with channel[] to specify a stop search   
  start = 0;				//tells wordsearch where to start searching for a specific word in tokens
  ending = tok_num; 			//""
  
  //Set the number of channels, if the number is not specified then it will be set to 1    	
  if(wordsearch("Channels",tokens,wordnumber,start))
    {
      ++wordnumber;
      start = wordnumber;
      info.insert(info.end(),tokens[wordnumber]);
      ++infocount;
      wordnumber= tok_num;
    }else
      {
	info.insert(info.end(),"1");
	++infocount;
	wordnumber= tok_num;
	start = 0;
      }
	
  //get the number of channel names by converting the string stored in tokens to an int
  int ChannelNum = atoi(info[0].c_str());
	
  if(mDebug>8)
    cout<<"Number of channels is "<<ChannelNum<<endl;
		
  //holds the position of each channel in tokens
  vector<int> ChannelPosition = GetPositionInfo(ChannelNum,start,tok_num,tokens,"Channel");
	
  if(mDebug == -5 || mDebug >9)
    {
      for(int i=0;i<ChannelNum;++i)
	{
	  cout<<"ChannelPosition " << i <<" = "<<ChannelPosition[i]<<endl;
	}
    }
	
  //______________________________________________Channel Loop__________________________________________	
  for(int a=0;a<ChannelNum;++a)
    {
      start = ChannelPosition[a];
      
      wordnumber = SetEndSearch(tok_num,ChannelNum,a,ChannelPosition);
       
      //_____________________________________________Check to see if config is in expert setup________________		
      if(flag=="expert" || flag == "Expert")
	{
	  int temp= ChannelPosition[a];
	  
	  info.insert(info.end(),tokens[temp]);//channelname
	  ++infocount;
	  ++temp;
	  
	  info.insert(info.end(),tokens[temp]);//number of bands
	  ++infocount;
	  
	  start = temp;
	  
	}else if(tokens[0] != "expert" || tokens[0] != "Expert"){	
	  if(mDebug>3)
	    {
	      cout<<"wordnumber is "<<wordnumber<<"at channelposition stuff"<<endl;
	    }
	    
	  if(wordsearch("Channelname",tokens,wordnumber,start))
	    {	
	      ++wordnumber;			
	      info.insert(info.end(),tokens[wordnumber]);
	      ++infocount;
	      
	      if(mDebug>3)
		{
		  cout<<"Channelname is "<<tokens[wordnumber]<<endl;
		}
	      
	    }else{
	      problem = "Channel name not defined";
	      cerr<<"Channel " <<a<<"name has not been defined in the config file"<<endl;
	      throw runtime_error(problem);
	      info.insert(info.end(),problem);
	      ++infocount;
	    }
  
	  wordnumber = SetEndSearch(tok_num,ChannelNum,a,ChannelPosition);
	  
	  //get the number of bands in Channel i+1(1,2 etc)
	  
	  if(wordsearch("Bands",tokens,wordnumber,start))
	    {
	      ++wordnumber;
	      info.insert(info.end(),tokens[wordnumber]);
	      ++infocount;
	      start = wordnumber;
	    }else
	      {
		info.insert(info.end(),"1");
		++infocount;
	      }
	}
            
      int BandNum = atoi(info[(infocount-1)].c_str());
      
      if(mDebug>4)
	{
	  cout<<"BANDNUM"<<endl;
	  cout<<tokens[wordnumber]<<endl;
	  cout<<"i="<<a<<endl;
	  cout<<"number of bands are "<<BandNum<<endl;
	}		
      
      if(mDebug>5)
	{
	  int tim = info.capacity();
	  cout<<"INFOCOUNT"<<endl<<"Capacity = "<<tim<<endl<<"number of elements = "<<infocount<<endl;
	}
      
      vector<int> BandPosition;
		
      //________________________________________________Band Loop_____________________________________________
       
      for(int i=0;i<BandNum;++i)
	{
	  
	  if(i == 0)
	    {
	      //cout<<"SetEndSearch ="<<SetEndSearch(tok_num,ChannelNum,a,ChannelPosition)<<endl;
	      BandPosition = GetPositionInfo(BandNum,start,SetEndSearch(tok_num,ChannelNum,a,ChannelPosition),tokens,"Band");
	      
	      /*for(int c = 0;c<BandNum;++c)
		{
		cout<<"bandposition = "<<BandPosition[c]<<endl;
		}*/
	    }			
	    
	  start = BandPosition[i];
	  int temp = SetEndSearch(tok_num,BandNum,i,BandPosition);
	  wordnumber = temp;	
	  
	  if(flag == "expert" || flag == "Expert")
	    {	
	      info.insert(info.end(),tokens[start]);
	      ++start;
	      ++infocount;
				
	      info.insert(info.end(),tokens[start]);
	      ++start;
	      ++infocount;
				
	      info.insert(info.end(),tokens[start]);
	      ++start;
	      ++infocount;
				
	      info.insert(info.end(),tokens[start]);
	      ++start;
	      ++infocount;
				
	      info.insert(info.end(),tokens[start]);
	      ++start;
	      ++infocount;
				
	    }else if(flag != "expert" || flag != "Expert")
	      {			
		
		if(wordsearch("Bandname",tokens,wordnumber,start))
		  {
		    ++wordnumber;
		    info.insert(info.end(),tokens[wordnumber]);//puts Bandname (e.g. Band1) into info vector
		    ++infocount;
		    start = wordnumber;
		  }else{
		        
		    // need to fix this ///////////////////////////////////////////////////////
		    ///////////////////////////////////////////////////////////////////////
		    ///////////////////////////////////////////////////////////////////////////
		    ofstream default_band;
		    ifstream get_crap;
		    string crap;
		    string default_name = "Band";
		    default_band.open("C:\\crap.txt");
		    default_band << (i+1);
		    get_crap.open("C:\\crap.txt");
		    getline(get_crap,crap);
		    default_name += crap;
		    info.insert(info.end(),default_name);
		    ++infocount;
		    start = BandPosition[i];				
		  }
		
		wordnumber = temp;			
		if(wordsearch("freqlo",tokens,wordnumber,start))
		  {
		    ++wordnumber;
		    info.insert(info.end(),tokens[wordnumber]);	//inserts the value of freqlo for Band i into info	
		    //start is not reset so that freqlo, freqhi, power_hi etc. can be interchanged
		    ++infocount;
		  }else{
		    cerr<<"Low Frequency for Band "<<i<<" is not defined."<<endl;
		    problem = "Low frequency not defined in configuration file";
		    throw runtime_error(problem);
		    info.insert(info.end(),problem);
		    ++infocount;
		  }
		
		wordnumber = temp;			
		if(wordsearch("freqhi",tokens,wordnumber,start))
		  {
		    ++wordnumber;
		    info.insert(info.end(),tokens[wordnumber]);
		    ++infocount;
		  }else{
		    cerr<<"High frequency for Band "<<i<< " is not defined"<<endl;
		    problem = "High frequency not defined in configuration file";
		    throw runtime_error(problem);
		    info.insert(info.end(),problem);
		    ++infocount;
		  }
		
		wordnumber = temp;			
		
		if(wordsearch("lopower",tokens,wordnumber,start))
		  {
		    ++wordnumber;
		    info.insert(info.end(),tokens[wordnumber]);
		    ++infocount;
		  }else{
		    cerr<<"Low Power for Band "<<(i+1)<< " is not defined"<<endl;
		    problem = "Low Power not defined in configuration file";
		    throw runtime_error(problem);
		    info.insert(info.end(),problem);
		    ++infocount;
		  }
				
		wordnumber = temp;			

		if(wordsearch("hipower",tokens,wordnumber,start))
		  {
		    ++wordnumber;
		    info.insert(info.end(),tokens[wordnumber]);
		    ++infocount;
		  }else{
		    cerr<<"High Power for Band "<<(i+1)<< " in Channel "<<a<< " is not defined"<<endl;
		    problem = "High Power not defined in configuration file";
		    throw runtime_error(problem);
		    info.insert(info.end(),problem);
		    ++infocount;
		  }		
	      }  
	}
    }
  
  return info;
}//END READCONFIG

vector<string> GetRequestedData(vector<string> total,int start, int ending)
{
  
  int mDebug =0;
  vector<string> requested;
  
  for(int i = start;i< ending;++i)
    {
      if(mDebug>6)
	cout<<total[i]<<endl;
      
      requested.insert(requested.end(),total[i]);// push_back(total[start]);
      /*for(int a=0;a<(ending +	1 - start);++a)
	{
	cout<<tokens[a]<<endl;
	}*/
    }
  return requested;
}

int SetEndSearch(int total,int variablenumber, int loopnumber,vector<int> position)
{
  int place =0;
  int mDebug=0;
  if(mDebug>8)
    {
      cout<<"total = "<<total<<endl<<
	"variablenumber = "<<variablenumber<<endl<<
	"loopnumber = "<<loopnumber<<endl;
    }
  
  if((variablenumber - loopnumber) != 1)
    {
      place = position[(loopnumber+1)];
    }else
      {
	place = total;
      }	
  return place;
}//END SETENDSEARCH

vector<int> GetPositionInfo(int num_var,int start_search,int vector_length,vector<string> tokens,string variable)
{
  int mDebug =0;
  vector<int> variable_vector;	//array that stores the values of where wordsearch should start for each channel, band,etc
  
  int i =0;
  int wordnumber= vector_length;	//variable to be passed by reference to wordsearch that will locate string place in the array,
  
  if(mDebug>5)
    {
      cout<<"num_var = "<<num_var<<endl<<"start_search = "<<start_search<<endl<<"vector_length = "<<vector_length<<endl<<
	"variable = "<<variable<<endl;
    }
   
  while(i < num_var)
    {
   
      string problem;
      
      if(wordsearch(variable,tokens,wordnumber,start_search))
	{
	  if(mDebug == -5)
	    {
	      cerr<< variable<<" "<<i << " was found"<<endl;
	    }
	  
	  ++wordnumber;
	  variable_vector.insert(variable_vector.end(),wordnumber);
	  start_search = wordnumber;
	  wordnumber = vector_length;
	  ++i;
	  
	}else
	  {
	    problem = variable + " has not been found: Please exit and fix the config file";
	    cerr<< variable<<" "<<i<<"has not been found";
	    throw runtime_error(problem);
	  }
    }
  
  return variable_vector;
}//END GETPOSITIONINFO

vector<string> createtokens(string line,int &tok_num)
{
  const string space = " \t="; //used to find the whitespace characters and endline and break up the line
  vector<string> token;
  size_t b =0;
  size_t beginning, ending;
  int mDebug=0;
  
  while(b < line.length())
    {	
      while(line[b] == space[0] || line[b] == space[2])
	{
	  ++b;
	}
      beginning=b;
      ++b;
      
      if(mDebug >1)
	cout<< "Beginning is "<<beginning<<endl;
      
      
      while(line[b] != space[0] && line[b] != space[1] && line[b] != space[2] && b < line.length())
	{
	  ++b;
	}
  
      ending = b;
      ++b;
      if(mDebug >1)
	cout<<"Ending is "<< ending<<endl;
     	
      ++tok_num;
      token.insert(token.end(),line.substr(beginning,ending-beginning));   
    }
  
  if(mDebug>0)
    cout<<"Number of tokens is "<<tok_num<<endl;
  
  return token;
}//END CREATETOKENS

bool wordsearch(string word,vector<string> stuff, int &span,int start)	//returns true if word is found in the line
{	
  bool found = false;
  
  
  for(int i = start;i<span;++i)
    {
      if(stuff[i]==word)
	{	
	  found = true;
	  span = i;
	  /*if(mDebug>0)
	    {
	    cerr<<"Word is located in spot "<<i<<"."<<endl;
	    }*/
	}
    }
  
  
  return found;
}//END WORDSEARCH

#endif
