/*----------------------------------------------------------------------*/
/*                                                         		*/
/* Module Name: LiveDataServer						*/
/* Module Description: DAQSocket wrapper for data retrieval		*/
/*                                                         		*/
/* Revision History:					   		*/
/* Rel   Date     Programmer  	Comments				*/
/* 0.1	 01Sep04  M. Evans   	First release				*/
/*                                                         		*/
/* see tdshead.hh for more information					*/
/*----------------------------------------------------------------------*/

#ifndef __LiveDataServer_hh__
#define __LiveDataServer_hh__

#include "DAQSocket.hh"
#include "LiveData.hh"

using namespace std;

//----------------------------------------------------------------------
// LiveDataServer
//----------------------------------------------------------------------

class LiveDataServer
{
  // -------- PUBLIC
  public:
  enum { BLOCKS_PER_SECOND = 16 };	// data block rate from NDS

  enum ResetLevel
  {
    NDS_SOCK,
    CHANNELS,
    SEQUENCE
  };

  enum DAQDataTypes		// FIX: this is wrong
  {
    SHORT = 1,
    INT = 2,
    LONG = 3,
    FLOAT = 4,
    DOUBLE = 5,
    QUAD = 6
  };

  LiveDataServer() { init(NDS_SOCK); }
  ~LiveDataServer() { clean(NDS_SOCK); }

  // connect to NDS and get list of available channels
  bool initNDS(const char* ndsName = NULL);

  // set channels for data request
  const LiveData* initChannels(const char** chanName, int numDataChan);

  // prepare to receive sequential data
  bool initDataFlow();

  // get a data block (stored in LiveData array returned by initChannels)
  bool getData(bool ignoreSeqNum = false);

  // some data access
  const DAQDRecHdr* getLastHeader() { return (DAQDRecHdr*)buf; }
  const LiveData* getChannels() { return chanList; }
  
  // -------- PROTECTED
  protected:
  DAQSocket nds;			// network data server

  int numAvl;				// number of available DAQ channels
  DAQDChannel* avlList;			// array of available DAQ channels

  int numChan;				// number of live data channels
  LiveData* chanList;			// array of live data channels

  int dataLen;				// expected data length
  int bufLen;				// communication buffer length
  char* buf;				// communication buffer

  int seqNum;				// last sequence number

  bool openNDS(const char* ndsName);	// open a socket to a NDS
  void checkBPS(DAQDChannel* chan, int size);	// check channel BPS

  void copyData();			// copy data from buf to chanList

  void init(ResetLevel lvl);		// initialize fields
  void clean(ResetLevel lvl);		// close socket to NDS and clean up
};

#endif // ifndef __LiveDataServer_hh__
