/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "SegAccountant.hh"
#include "Segment.hh"
#include "TrigClient.hh"
#include <stdexcept>

using namespace std;
using namespace trig;

//======================================  seg_info constructor
SegAccountant::seg_info::seg_info(const string& name, int versn) 
    : _segname(name), _version(versn), _start(0), _last(0), _state(false),
      _mode(kNone), _delay(0.0)
{}

//======================================  seg_info constructor
SegAccountant::seg_info::~seg_info(void) {
}

//======================================  set flosh mode
void
SegAccountant::seg_info::set_mode(flush_mode m, Interval t) {
    _mode  = m;
    _delay = t;
}

//======================================  Construct segment accountant
SegAccountant::SegAccountant(void) 
    : mTC(0)
{
}

//======================================  Construct segment accountant
SegAccountant::SegAccountant(TrigClient& tc) 
    : mTC(&tc), mSegStart(0)
{
}

//======================================  Destroy segment accountant
SegAccountant::~SegAccountant(void) {
}

//======================================  Add a segment type to be managed
void 
SegAccountant::addSegment(const string& name, int version, 
			  flush_mode mode, Interval dly) {
    seg_info si(name, version);
    si.set_mode(mode, dly);
    unsigned int insp = locate(name);
    if (insp < mList.size() && (mList[insp].name() == name)) {
	string msg("addSegment called for existing segment type: ");
	msg += name;
	throw runtime_error(msg);
    }
    mList.insert(mList.begin()+insp, si);
}

//======================================  Find the named segment type info
SegAccountant::seg_info& 
SegAccountant::find(const string& str) {
    int inx = locate(str);
    if (mList[inx].name() != str) {
	throw runtime_error("SegAccountant: Named segment not found");
    }
    return mList[inx];
}

//======================================  Locate the named segment type
int
SegAccountant::locate(const string& str) {
    int hi = mList.size();
    int lo = 0;
    while (hi > lo) {
	int mid = (hi+lo+1) >> 1;
	if (str < mList[mid].name()) hi = mid;
	else                         lo = mid;
    }
    return hi;
}

//======================================  Locate the named segment type
void 
SegAccountant::start_stride(const Time& ts) {
    ;
}

//======================================  Locate the named segment type
void 
SegAccountant::update(const Time& tup) {
    ;
}
